@@ -7,15 +7,20 @@ __author__ = "Jared Beard"
 
 import sys
 import os
+
+from traitlets import dlink
 current = os.path.dirname(os.path.realpath(__file__))
 parent = os.path.dirname(current)
 sys.path.append(parent)
 
+from collections.abc import Iterable
 from copy import deepcopy
 
-__all__ = ["merge", "unstructure", "structure", "find_key", "recursive_set"]
+import itertools
 
-def merge(d_default : dict, d_merge : dict):
+__all__ = ["merge", "unstructure", "structure", "find_key", "recursive_set", "recursive_get"]
+
+def merge(d_default : dict, d_merge : dict, do_append : bool = False):
     """
     Adds d_merge values to d_default recursively. 
     Values in d_merge overwrite those of d_default values, 
@@ -24,14 +29,21 @@ def merge(d_default : dict, d_merge : dict):
 
     :param d_default: (dict) base dictionary
     :param d_merge: (dict) dictionary to add
+    :param do_append: (bool) if true, will append iterable elements of not dict type, *default*: Flase
     :return: (dict) combined dictionary
     """
     d_default = deepcopy(d_default)
     for key in d_merge:
-        if key not in d_default or not (isinstance(d_merge[key], dict) and isinstance(d_default[key], dict)):
+        if key not in d_default: 
             d_default[key] = deepcopy(d_merge[key])
         else:
-            d_default[key] = merge(d_default[key], d_merge[key])
+            if isinstance(d_default[key],dict) and isinstance(d_merge[key],dict):
+                d_default[key] = merge(d_default[key], d_merge[key], do_append)
+            elif isinstance(d_merge[key], Iterable) and type(d_default[key]) == type(d_merge[key]):
+                d_default[key] += d_merge[key]
+            else:
+                d_default[key] = deepcopy(d_merge[key])
+
     return d_default
     
 def unstructure(d):
@@ -93,13 +105,43 @@ def find_key(d : dict, key):
     else:
         return [key]
     return None
+
+def find_all_keys(d : dict, key):
+    """
+    Finds all instances of key in nested dict
+    
+    :param d: (dict) dictionary to search
+    :param key: () key
+    :return: (list) Returns list of keys
+    """
+    keys = []
+    for k in d:
+        if k == key:
+            keys.append(key)
+        else:
+            if isinstance(d[k], dict):
+                temp_keys = find_all_keys(d[k],key)
+                for el in temp_keys:
+                    keys.append(k + el)
+    return keys
+
+def expand_keys(keys : list):
+    """
+    Given a key where some elements are shared, expands to a list of keys.
+    
+    For example [var1, [var2, var3]] would become the keys [var1, var2] and [var1, var3].
+    
+    :param key: (list) compressed keys
+    :return: (list) listed keys
+    """
+    return list(itertools.product(*keys)) 
     
 def recursive_set(d : dict, key : list, val, as_hint = False):
     """
     Updates dictionary value given an ordered list of keys.
     Can also support keys as hints and will search for the *first* key before attempting to set it.
     (Later may update find_key to match a list of keys or make a find_key_list function)
-    In either case, if key is not found it will be added to root
+    In either case, if key is not found it will be added to root.
     
     :param d: (dict) dictionary to update
     :param key: (list) list of keys
@@ -117,4 +159,19 @@ def recursive_set(d : dict, key : list, val, as_hint = False):
                 d[key[0]] = {}
             recursive_set(d[key[0]], key[1:len(key)], val)
         else:
-            d[key[0]] = val
\ No newline at end of file
+            d[key[0]] = val
+            
+def recursive_get(d : dict, key : list):
+    """
+    Returns value from dict recursive key
+    
+    :param d: (dict) dictionary containing key
+    :param key: (list) list of keys
+    :return: () Value
+    """
+    if not isinstance(key,list):
+        return d[key]
+    if len(key) == 1:
+        return d[key[0]]
+    return recursive_get(d[key[0]],key[1:len(key)])
+
