@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 from copy import deepcopy
 from nestifydict import *
 
@@ -17,6 +18,22 @@ print("b <- a", merge(b,a))
 print("c <- b", merge(c,b))
 print("c <- a", merge(c,a))
 
+## Test Merge w/ Append---------------------------
+#Test with merge list, set, and dict
+
+d = {"a": 1, "b": [1,2,3]}
+e = {"a": 1, "b": [4,5]}
+f = {"a": 1, "b": (4,5)}
+g = {"a": 1, "b": {"c":1}}
+print("d", d)
+print("e", e)
+print("f", f)
+print("g", g)
+print("d <- e", merge(d,e))
+print("d <- f", merge(d,f))
+print("d <- g", merge(d,g))
+print("f <- f", merge(f,f))
+
 ## Test Unstructure ------------------------------
 print("\n------ Unstructure ------")
 
@@ -51,4 +68,10 @@ print("\n------ Recurcsive Set ------")
 recursive_set(a,a1akey,"hi")
 print("set a1a1 full key", a)
 recursive_set(a,["a1a"],"bye", True)
-print("set a1a1 as hint", a)
\ No newline at end of file
+print("set a1a1 as hint", a)
+
+## Test Recursive Get ----------------------------
+print("\n------ Recurcsive Get ------")
+
+
+print("get a1a1 full key", a1akey, " -> ", recursive_get(a,a1akey))
